package com.localtransfer;

import android.content.ContentUris;
import android.content.Context;
import android.database.Cursor;
import android.net.Uri;
import android.provider.MediaStore;
import android.util.Log;
import android.widget.Button;
import android.widget.LinearLayout;

import com.google.android.material.snackbar.Snackbar;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;

public class ServerFile extends Transfer {

    private Integer id;

    public String name;

    public String href;

    public String mime;

    public String type;

    public long size;

    public String save_location;

    public Uri uri;

    public Button button;

    public Progress progress;

    private static List<ServerFile> instances = new ArrayList<>();

    public ServerFile(Integer id) {
        this.id = id;
        instances.add(this);
    }

    public static List getInstances() {
        return instances;
    }

    public static ServerFile getFileById(int id) {
        for (Object obj: instances) {
            ServerFile p = (ServerFile) obj;
            if (p.id == id) return p;
        }
        return null;
    }

    public Integer getId() {
        return id;
    }

    public boolean exist(Context context) {
        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.Q) {
            Cursor cursor = context.getContentResolver().query(MediaStore.Downloads.EXTERNAL_CONTENT_URI, null, null, null, null);
            while (cursor.moveToNext()) {
                String MediaStore_File_name = cursor.getString(cursor.getColumnIndexOrThrow(MediaStore.Downloads.DISPLAY_NAME));
                long MediaStore_File_size = cursor.getLong(cursor.getColumnIndexOrThrow(MediaStore.Downloads.SIZE));
                if(MediaStore_File_name.equals(name) && MediaStore_File_size == size) {
                    int cursor_id = cursor.getInt(cursor.getColumnIndexOrThrow(MediaStore.Downloads._ID));
                    uri = ContentUris.withAppendedId(MediaStore.Downloads.EXTERNAL_CONTENT_URI, cursor_id);
                    return true;
                }
            }
        }
        else {
            File file = new File(save_location, name);
            new File(save_location).mkdirs();
            uri = Uri.fromFile(file);
            if (file.exists() && file.length() == size)
                return true;
        }
        return false;
    }

    public InputStream getThumbnail() {
        InputStream thumbnail = null;
        URL url;
        try {
            String query = URLEncoder.encode(href, StandardCharsets.UTF_8.toString());
            String serverUrl = root + "/thumbnail.php?file=" + query;
            url = new URL(protocol, host, port, serverUrl);
            HttpURLConnection conn = (HttpURLConnection) url.openConnection();
            conn.setRequestMethod("GET");
            conn.setConnectTimeout(2000);

            conn.connect();

            thumbnail = conn.getInputStream();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return thumbnail;
    }

    public void downloadFile() throws IOException {

        URL url = null;
        HttpURLConnection conn = null;
        int maxBufferSize = 1 * 1024 * 1024;
        byte[] buffer = new byte[maxBufferSize];
        int bufferLength;
        long loaded = 0;

        String[] parts = href.split("/");
        String path = "";
        for(int i=0; i< parts.length - 1; i++) {
            path = path + parts[i] + "/";
        }
        String query = URLEncoder.encode(parts[parts.length - 1], StandardCharsets.UTF_8.toString());
        url = new URL(protocol, host, port, path + query.replace("+", "%20"));
        conn = (HttpURLConnection) url.openConnection();
        conn.setDoOutput(true);
        conn.setRequestMethod("GET");
        conn.setConnectTimeout(2000);

        Log.d("URL", url.toString());

        conn.connect();

        int responseCode = conn.getResponseCode();

        if (responseCode == HttpURLConnection.HTTP_OK) {

            OutputStream fileOutput = activity.getContentResolver().openOutputStream(uri);
            InputStream in = conn.getInputStream();

            while ((bufferLength = in.read(buffer)) > 0) {
                fileOutput.write(buffer, 0, bufferLength);
                loaded+= (long) bufferLength;
                progress.loaded = loaded;
                progress.percent = ((loaded * 100) / size);
                if(Progress.app_started && progress.button != null)
                    activity.runOnUiThread(() -> progress.button.setText(String.format("Download in progress %d%%", progress.percent)));
            }
            fileOutput.close();

            conn.disconnect();

            progress.stopProgress();

            if(progress.button != null) {
                activity.runOnUiThread(() -> {
                    final LinearLayout layout = (LinearLayout) progress.button.getParent();
                    layout.findViewById(R.id.file_view).setVisibility(LinearLayout.VISIBLE);
                    layout.findViewById(R.id.file_share).setVisibility(LinearLayout.VISIBLE);
                    progress.button.setVisibility(LinearLayout.GONE);
                    progress.button.setEnabled(true);

                    progress.button.setText(activity.getString(R.string.file_download));
                    progress.button.setCompoundDrawables(progress.button.getCompoundDrawables()[0], null, null, null);
                });
            }
            activity.runOnUiThread(() -> {
                String message = "File " + name + " successful download";
                System.out.println(message);
                Snackbar.make(activity.findViewById(R.id.view_pager), message, Snackbar.LENGTH_LONG)
                        .setAction("Action", null).show();
            });
        }
        else {
            String s = conn.getResponseMessage();
            throw new HttpErrorException("error code: " + responseCode + " " + s);
        }

        conn.disconnect();
    }

    public String deleteFile() throws IOException {

        URL url = null;
        HttpURLConnection conn = null;

        String message;

        String query = URLEncoder.encode(name, StandardCharsets.UTF_8.toString());
        String serverUrl = root + "/delete.php?file=" + query;
        url = new URL(protocol, host, port, serverUrl);
        conn = (HttpURLConnection) url.openConnection();
        conn.setRequestMethod("GET");
        conn.setConnectTimeout(2000);

        conn.connect();

        int responseCode = conn.getResponseCode();

        if (responseCode == HttpURLConnection.HTTP_OK) {

            message = "File " + name + " successful deleted";
        }
        else {
            String s = conn.getResponseMessage();
            throw new HttpErrorException("error code: " + responseCode + " " + s);
        }

        conn.disconnect();

        return message;
    }
}
