package com.localtransfer.fragment;

import android.animation.ObjectAnimator;
import android.content.ActivityNotFoundException;
import android.content.ContentValues;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.os.Environment;
import android.provider.MediaStore;
import android.view.ContextMenu;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.Animation;
import android.view.animation.LinearInterpolator;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.fragment.app.Fragment;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import com.google.android.material.snackbar.Snackbar;
import com.localtransfer.Progress;
import com.localtransfer.R;
import com.localtransfer.Transfer;
import com.localtransfer.ServerFile;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

public class DownloadFragment extends Fragment {

    public DownloadFragment() {
        // Required empty public constructor
    }

    private View root;
    private LayoutInflater inflater;

    public static DownloadFragment newInstance() {
        DownloadFragment fragment = new DownloadFragment();
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        root = inflater.inflate(R.layout.fragment_download, container, false);

        this.inflater = inflater;

        new Timer().schedule(new TimerTask() {
            @Override
            public void run() {
                listFile();
            }
        }, 100);

        SwipeRefreshLayout refresh = root.findViewById(R.id.refresh);


        refresh.setOnRefreshListener(() -> {
            listFile();
            refresh.setRefreshing(false);
        });

        return root;
    }

    @Override
    public void onPause(){
        super.onPause();

    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();

    }

    @Override
    public void onCreateContextMenu(ContextMenu menu, View v, ContextMenu.ContextMenuInfo menuInfo) {
        super.onCreateContextMenu(menu, v, menuInfo);
        menu.add ( 0 , v.getId (), 0 , "Delete" );
    }

    @Override
    public boolean onContextItemSelected(MenuItem item) {
        if (item.getTitle() == "Delete") {
            LinearLayout fileDesc = (LinearLayout) root.findViewById(item.getItemId());
            ServerFile file = ServerFile.getFileById((Integer) fileDesc.getTag(R.id.ID_DOWNLOAD));
            new Thread(() -> {
                try {
                    String message = file.deleteFile();
                    System.out.println(message);
                    getActivity().runOnUiThread(() ->
                            Snackbar.make(getActivity().findViewById(R.id.view_pager), message, Snackbar.LENGTH_LONG)
                                    .setAction("Action", null).show());
                    listFile();
                } catch (IOException e) {
                    String ExceptionName = e.getClass().getSimpleName();
                    String ExceptionMess = e.getMessage();

                    if(ExceptionName != null && ExceptionMess != null) {
                        Transfer.error(ExceptionName + ": " + ExceptionMess, null, null);
                    }

                }
            }).start();
        }
        else {
            return  false;
        }
        return true;
    }

    public void listFile() {
        new Thread(() -> {
            final TextView title = new TextView(Transfer.activity);
            final LinearLayout main_layout = root.findViewById(R.id.main_layout);
            try {
                String data = Transfer.getFileList();

                if (data.equals("[]")) {
                    title.setTextSize(18);
                    title.setGravity(Gravity.CENTER);
                    main_layout.removeAllViews();
                    main_layout.addView(title);
                    title.setHeight(1000);
                    title.setText("No file to download");
                }
                else {

                    getActivity().runOnUiThread(() -> {
                        main_layout.removeAllViews();
                        final View line = inflater.inflate(R.layout.horizontal_line, null);
                        main_layout.addView(line);
                    });

                    try {
                        JSONArray array = new JSONArray(data);
                        for (int i = 0; i < array.length(); i++) {
                            Integer id = View.generateViewId();
                            ServerFile trFile = new ServerFile(id);

                            JSONObject row = array.getJSONObject(i);
                            trFile.name = row.getString("name");
                            trFile.size = row.getLong("size");
                            trFile.href = row.getString("href");
                            trFile.mime = row.getString("mime");
                            trFile.type = row.getString("type");

                            getActivity().runOnUiThread(() -> showFileList(trFile));

                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                    }
                }
            } catch (IOException e) {
                final String ExceptionName = e.getClass().getSimpleName();
                final String ExceptionMess = e.getMessage();

                if(ExceptionName != null && ExceptionMess != null) {
                    Transfer.error(ExceptionName + ": " + ExceptionMess, title, main_layout);
                }

            }
        }).start();
    }

    private void setThumbnail(ServerFile file, ImageView image) {
        new Thread(() -> {
            InputStream thumbnail = file.getThumbnail();
            Bitmap bitmap = BitmapFactory.decodeStream(thumbnail);
            getActivity().runOnUiThread(() -> {
                image.setImageBitmap(bitmap);
            });
        }).start();
    }

    private void showFileList(ServerFile trFile) {

        final LinearLayout main_layout = root.findViewById(R.id.main_layout);

        View file_info = inflater.inflate(R.layout.file_info, null);
        main_layout.addView(file_info);
        ImageView image = file_info.findViewById(R.id.file_image);
        TextView viewName = file_info.findViewById(R.id.file_name);
        TextView viewType = file_info.findViewById(R.id.file_type);
        TextView viewSize = file_info.findViewById(R.id.file_size);
        LinearLayout fileDesc = file_info.findViewById(R.id.file_desc);
        LinearLayout file_buttons = file_info.findViewById(R.id.file_buttons);

        fileDesc.setId(View.generateViewId());

        registerForContextMenu(fileDesc);

        file_buttons.setId(trFile.getId());
        file_buttons.setTag(R.id.ID_DOWNLOAD, trFile.getId());
        fileDesc.setTag(R.id.ID_DOWNLOAD, trFile.getId());
        file_buttons.setTag(R.id.ID_FILE_BUTTONS, "FOR VISIBILITY");

        trFile.button = file_buttons.findViewById(R.id.file_download);

        if (trFile.mime.equals("text/plain"))
            trFile.save_location = getContext().getCacheDir().toString();
        else
            trFile.save_location = Environment.getExternalStorageDirectory() + "/" + Transfer.local_storage;

        viewName.setText(trFile.name);
        viewType.setText(trFile.mime);
        viewSize.setText(Transfer.humanReadableByteCountBin(trFile.size));

        switch (trFile.type) {
            case "file-image":
                image.setImageResource(R.drawable.ic_icon_image);
                setThumbnail(trFile, image);
                break;
            case "file-video":
                image.setImageResource(R.drawable.ic_icon_video);
                setThumbnail(trFile, image);
                break;
            case "file-audio":
                image.setImageResource(R.drawable.ic_icon_music);
                break;
            default:
                image.setImageResource(R.drawable.ic_icon_file);
        }

        fileDesc.setOnClickListener(view -> {

            Button Bview = file_buttons.findViewById(R.id.file_view);
            Button Bshare = file_buttons.findViewById(R.id.file_share);
            Button Bdownload = file_buttons.findViewById(R.id.file_download);

            Bdownload.setOnClickListener(ListenerDL);
            Bview.setOnClickListener(ListenerView);
            Bshare.setOnClickListener(ListenerShare);

            if (trFile.exist(getContext())) {
                Bview.setVisibility(LinearLayout.VISIBLE);
                Bshare.setVisibility(LinearLayout.VISIBLE);
                Bdownload.setVisibility(LinearLayout.GONE);
            }
            else {
                Bdownload.setVisibility(LinearLayout.VISIBLE);
                Bview.setVisibility(LinearLayout.GONE);
                Bshare.setVisibility(LinearLayout.GONE);
            }

            if (Progress.setButton(trFile.name, Bdownload)) {
                Bdownload.setEnabled(false);
                final Drawable spinner = Transfer.activity.getDrawable(R.drawable.ic_spinner_rotate);
                int h = spinner.getIntrinsicHeight();
                int w = spinner.getIntrinsicWidth();
                spinner.setBounds(0, 0, w, h);
                Bdownload.setCompoundDrawables(Bdownload.getCompoundDrawables()[0], null, spinner, null);
                ObjectAnimator anim = ObjectAnimator.ofInt(spinner, "level", 0, 10000);
                anim.setDuration(1000);
                anim.setRepeatCount(Animation.INFINITE);
                anim.setInterpolator(new LinearInterpolator());
                anim.start();
            }

            int state = file_buttons.getVisibility();
            if(state == LinearLayout.GONE) {
                for(View child : getAllChildren(main_layout)) {
                    if(child.getTag(R.id.ID_FILE_BUTTONS) != null)
                        child.setVisibility(LinearLayout.GONE);
                }
                file_buttons.setVisibility(LinearLayout.VISIBLE);
            }
            else if(state == LinearLayout.VISIBLE)
                file_buttons.setVisibility(LinearLayout.GONE);
        });
    }

    private View.OnClickListener ListenerDL = v -> {
        v.setEnabled(false);
        final LinearLayout layout = (LinearLayout) v.getParent();
        final Button button = layout.findViewById(R.id.file_download);

        final Integer id = (Integer) layout.getTag(R.id.ID_DOWNLOAD);

        final Drawable image = getActivity().getDrawable(R.drawable.ic_spinner_rotate);
        int h = image.getIntrinsicHeight();
        int w = image.getIntrinsicWidth();
        image.setBounds(0, 0, w, h);
        button.setCompoundDrawables(button.getCompoundDrawables()[0], null, image, null);
        ObjectAnimator anim = ObjectAnimator.ofInt(image, "level", 0, 10000);
        anim.setDuration(1000);
        anim.setRepeatCount(Animation.INFINITE);
        anim.setInterpolator(new LinearInterpolator());
        anim.start();

        ServerFile file = ServerFile.getFileById(id);

        file.progress = new Progress(file.name, file.size, Progress.DOWNLOAD);
        file.progress.button = file.button;

        if (file.uri == null && android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.Q) {
            ContentValues values = new ContentValues();
            values.put(MediaStore.MediaColumns.DISPLAY_NAME, file.name);
            values.put(MediaStore.MediaColumns.MIME_TYPE, file.mime);
            values.put(MediaStore.MediaColumns.RELATIVE_PATH, Transfer.local_storage);
            file.uri = getContext().getContentResolver().insert(MediaStore.Downloads.EXTERNAL_CONTENT_URI, values);
        }

        new Thread(() -> {
            try {
                file.downloadFile();
            } catch (IOException e) {
                final String ExceptionName = e.getClass().getSimpleName();
                final String ExceptionMess = e.getMessage();

                if(ExceptionName != null && ExceptionMess != null) {
                    Transfer.error(ExceptionName + ": " + ExceptionMess, null, null);
                }

            }
        }).start();
    };

    private View.OnClickListener ListenerShare = v -> {
        LinearLayout layout = (LinearLayout) v.getParent();
        final Integer id = (Integer) layout.getTag(R.id.ID_DOWNLOAD);

        ServerFile dl = ServerFile.getFileById(id);

        if(dl.type.equals("text/plain")) {
            String text = null;
            try {
                InputStream is = getActivity().getContentResolver().openInputStream(dl.uri);
                BufferedReader buf = new BufferedReader(new InputStreamReader(is));

                String line;
                StringBuilder sb = new StringBuilder();

                while((line = buf.readLine()) != null) {
                    sb.append(line).append("\n");
                }

                text = sb.toString();
            } catch (IOException e) {
                e.printStackTrace();
            }


            try {
                Intent intent = new Intent(Intent.ACTION_SEND);
                intent.setType(dl.type);
                intent.putExtra(Intent.EXTRA_TEXT, text);
                startActivity(Intent.createChooser(intent, getString(R.string.share_title)));
            } catch (ActivityNotFoundException e) {
                e.printStackTrace();
            }
        }
        else {
            try {
                Intent intent = new Intent(Intent.ACTION_SEND);
                intent.putExtra(Intent.EXTRA_STREAM, dl.uri);
                intent.setType(dl.mime);
                intent.setFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
                startActivity(Intent.createChooser(intent, getString(R.string.share_title)));
            } catch (ActivityNotFoundException e) {
                e.printStackTrace();
            }
        }


    };

    private View.OnClickListener ListenerView = v -> {
        LinearLayout layout = (LinearLayout) v.getParent();
        final Integer id = (Integer) layout.getTag(R.id.ID_DOWNLOAD);

        ServerFile dl = ServerFile.getFileById(id);

        try {
            Intent intent = new Intent(Intent.ACTION_VIEW);
            intent.setDataAndType(dl.uri, dl.mime);
            intent.setFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
            startActivity(intent);
        } catch (ActivityNotFoundException e) {
            e.printStackTrace();
        }


    };

    private List<View> getAllChildren(View v) {

        if (!(v instanceof ViewGroup)) {
            ArrayList<View> viewArrayList = new ArrayList<>();
            viewArrayList.add(v);
            return viewArrayList;
        }

        ArrayList<View> result = new ArrayList<>();

        ViewGroup viewGroup = (ViewGroup) v;
        for (int i = 0; i < viewGroup.getChildCount(); i++) {

            View child = viewGroup.getChildAt(i);

            //add child
            result.add(child);

            //Do the same with child of child
            result.addAll(getAllChildren(child));
        }
        return result;
    }

}